/*
  Sample program illustrating x86 assembly code used
  inside a C program and referencing C variables.
*/
#include <stdio.h>

int main( )
{
   /*
      Variables to be referenced MUST be static and have an assembly
      variable name string declared.  The assembly string does NOT
      have to be the same as the "C" variable name.
   */
   static int a asm ("a") = 1234;
   static int b asm ("b") = 5678;
   printf("a = %d, b = %d\n", a, b);
   /*
     The assembly code is included as a "function call":

        asm volatile(<assembly code as a single string>);

     Where "asm" may be
        asm
        __asm
        __asm__

     and where "volatile" may be
        volatile
        __volatile
        __volatile__
   */
   __asm__ __volatile__(
      "pushq %rax       \n"
      "pushq %rbx       \n"
      "movl  a,   %rax  \n"
      "movl  b,   %rbx  \n"
      "movl %rbx, a     \n"
      "movl %rax, b     \n"
      "popq %rbx        \n"
      "popq %rax        \n"
   );
   /*
     Here the C code continues.
   */
   printf("a = %d, b = %d\n", a, b);
   return(0);
}
